@@ -29,16 +29,18 @@
 #include <ctype.h>
 #include <limits.h>
 #include <uuid/uuid.h>
+#include "kernel-shared/uapi/btrfs.h"
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/disk-io.h"
 #include "kernel-shared/volumes.h"
 #include "common/utils.h"
+#include "common/device-utils.h"
 #include "common/path-utils.h"
 #include "common/open-utils.h"
+#include "common/sysfs-utils.h"
 #include "common/messages.h"
 #include "cmds/commands.h"
 #include "mkfs/common.h"
-#include "ioctl.h"
 
 static int rand_seed_initialized = 0;
 static unsigned short rand_seed[3];
@@ -66,18 +68,6 @@ void btrfs_format_csum(u16 csum_type, const u8 *data, char *output)
 	}
 }
 
-int get_device_info(int fd, u64 devid,
-		struct btrfs_ioctl_dev_info_args *di_args)
-{
-	int ret;
-
-	di_args->devid = devid;
-	memset(&di_args->uuid, '\0', sizeof(di_args->uuid));
-
-	ret = ioctl(fd, BTRFS_IOC_DEV_INFO, di_args);
-	return ret < 0 ? -errno : 0;
-}
-
 int get_df(int fd, struct btrfs_ioctl_space_args **sargs_ret)
 {
 	u64 count = 0;
@@ -230,7 +220,7 @@ int get_fs_info(const char *path, struct btrfs_ioctl_fs_info_args *fi_args,
 			goto out;
 		}
 		ret = check_mounted_where(fd, path, mp, sizeof(mp),
-					  &fs_devices_mnt, SBREAD_DEFAULT);
+					  &fs_devices_mnt, SBREAD_DEFAULT, false);
 		if (!ret) {
 			ret = -EINVAL;
 			goto out;
@@ -280,7 +270,7 @@ int get_fs_info(const char *path, struct btrfs_ioctl_fs_info_args *fi_args,
 		 * search_chunk_tree_for_fs_info() will lacks the devid 0
 		 * so manual probe for it here.
 		 */
-		ret = get_device_info(fd, 0, &tmp);
+		ret = device_get_info(fd, 0, &tmp);
 		if (!ret) {
 			fi_args->num_devices++;
 			ndevs++;
@@ -303,7 +293,7 @@ int get_fs_info(const char *path, struct btrfs_ioctl_fs_info_args *fi_args,
 		memcpy(di_args, &tmp, sizeof(tmp));
 	for (; last_devid <= fi_args->max_id && ndevs < fi_args->num_devices;
 	     last_devid++) {
-		ret = get_device_info(fd, last_devid, &di_args[ndevs]);
+		ret = device_get_info(fd, last_devid, &di_args[ndevs]);
 		if (ret == -ENODEV)
 			continue;
 		if (ret)
@@ -498,7 +488,7 @@ static bool valid_escape(const char *str)
  * - line is advanced to the final separator or nul character
  * - returned path is a valid string terminated by zero or whitespace separator
  */
-char *read_path(char **line)
+static char *read_path(char **line)
 {
 	char *ret = *line;
 	char *out = *line;
@@ -1160,78 +1150,6 @@ void btrfs_warn_experimental(const char *str)
 #endif
 }
 
-/*
- * Open a file in fsid directory in sysfs and return the file descriptor or
- * error
- */
-int sysfs_open_fsid_file(int fd, const char *filename)
-{
-	u8 fsid[BTRFS_UUID_SIZE];
-	char fsid_str[BTRFS_UUID_UNPARSED_SIZE];
-	char sysfs_file[PATH_MAX];
-	int ret;
-
-	ret = get_fsid_fd(fd, fsid);
-	if (ret < 0)
-		return ret;
-	uuid_unparse(fsid, fsid_str);
-
-	ret = path_cat3_out(sysfs_file, "/sys/fs/btrfs", fsid_str, filename);
-	if (ret < 0)
-		return ret;
-
-	return open(sysfs_file, O_RDONLY);
-}
-
-/*
- * Open a file in the toplevel sysfs directory and return the file descriptor
- * or error.
- */
-int sysfs_open_file(const char *name)
-{
-	char path[PATH_MAX];
-	int ret;
-
-	ret = path_cat_out(path, "/sys/fs/btrfs", name);
-	if (ret < 0)
-		return ret;
-	return open(path, O_RDONLY);
-}
-
-/*
- * Open a directory by name in fsid directory in sysfs and return the file
- * descriptor or error, filedescriptor suitable for fdreaddir. The @dirname
- * must be a directory name.
- */
-int sysfs_open_fsid_dir(int fd, const char *dirname)
-{
-	u8 fsid[BTRFS_UUID_SIZE];
-	char fsid_str[BTRFS_UUID_UNPARSED_SIZE];
-	char sysfs_file[PATH_MAX];
-	int ret;
-
-	ret = get_fsid_fd(fd, fsid);
-	if (ret < 0)
-		return ret;
-	uuid_unparse(fsid, fsid_str);
-
-	ret = path_cat3_out(sysfs_file, "/sys/fs/btrfs", fsid_str, dirname);
-	if (ret < 0)
-		return ret;
-
-	return open(sysfs_file, O_DIRECTORY | O_RDONLY);
-}
-
-/*
- * Read up to @size bytes to @buf from @fd
- */
-int sysfs_read_file(int fd, char *buf, size_t size)
-{
-	lseek(fd, 0, SEEK_SET);
-	memset(buf, 0, size);
-	return read(fd, buf, size);
-}
-
 static const char exclop_def[][16] = {
 	[BTRFS_EXCLOP_NONE]		= "none",
 	[BTRFS_EXCLOP_BALANCE]		= "balance",
