@@ -507,10 +507,10 @@ static void list_all_devices(struct btrfs_root *root)
 	printf("\n");
 }
 
-static int is_temp_block_group(struct extent_buffer *node,
-			       struct btrfs_block_group_item *bgi,
-			       u64 data_profile, u64 meta_profile,
-			       u64 sys_profile)
+static bool is_temp_block_group(struct extent_buffer *node,
+				struct btrfs_block_group_item *bgi,
+				u64 data_profile, u64 meta_profile,
+				u64 sys_profile)
 {
 	u64 flag = btrfs_block_group_flags(node, bgi);
 	u64 flag_type = flag & BTRFS_BLOCK_GROUP_TYPE_MASK;
@@ -537,26 +537,26 @@ static int is_temp_block_group(struct extent_buffer *node,
 	 * So only use condition 1) and 2) to judge them.
 	 */
 	if (used != 0)
-		return 0;
+		return false;
 	switch (flag_type) {
 	case BTRFS_BLOCK_GROUP_DATA:
 	case BTRFS_BLOCK_GROUP_DATA | BTRFS_BLOCK_GROUP_METADATA:
 		data_profile &= BTRFS_BLOCK_GROUP_PROFILE_MASK;
 		if (flag_profile != data_profile)
-			return 1;
+			return true;
 		break;
 	case BTRFS_BLOCK_GROUP_METADATA:
 		meta_profile &= BTRFS_BLOCK_GROUP_PROFILE_MASK;
 		if (flag_profile != meta_profile)
-			return 1;
+			return true;
 		break;
 	case BTRFS_BLOCK_GROUP_SYSTEM:
 		sys_profile &= BTRFS_BLOCK_GROUP_PROFILE_MASK;
 		if (flag_profile != sys_profile)
-			return 1;
+			return true;
 		break;
 	}
-	return 0;
+	return false;
 }
 
 /* Note: if current is a block group, it will skip it anyway */
@@ -990,7 +990,7 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 	struct btrfs_root *root;
 	struct btrfs_fs_info *fs_info;
 	struct btrfs_trans_handle *trans;
-	struct open_ctree_flags ocf = { 0 };
+	struct open_ctree_args oca = { 0 };
 	int ret = 0;
 	int close_ret;
 	int i;
@@ -1569,9 +1569,9 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 		goto error;
 	}
 
-	ocf.filename = file;
-	ocf.flags = OPEN_CTREE_WRITES | OPEN_CTREE_TEMPORARY_SUPER;
-	fs_info = open_ctree_fs_info(&ocf);
+	oca.filename = file;
+	oca.flags = OPEN_CTREE_WRITES | OPEN_CTREE_TEMPORARY_SUPER;
+	fs_info = open_ctree_fs_info(&oca);
 	if (!fs_info) {
 		error("open ctree failed");
 		goto error;
