@@ -48,12 +48,27 @@ check_prereq btrfs
 check_prereq btrfstune
 check_kernel_support
 
+test_found=0
+
 run_one_test() {
+	local testdir
 	local testname
 
-	testname="$1"
+	testdir="$1"
+	testname=$(basename "$testdir")
+	if ! [ -z "$TEST_FROM" ]; then
+		if [ "$test_found" == 0 ]; then
+			case "$testname" in
+				$TEST_FROM) test_found=1;;
+			esac
+		fi
+		if [ "$test_found" == 0 ]; then
+			printf "    [TEST/fsck]   %-32s (SKIPPED)\n" "$testname"
+			return
+		fi
+	fi
 	echo "    [TEST/fsck]   $(basename $testname)"
-	cd "$testname"
+	cd "$testdir"
 	echo "=== START TEST $testname" >> "$RESULTS"
 	if [ -x test.sh ]; then
 		# Type 2
@@ -62,9 +77,9 @@ run_one_test() {
 			if [[ $TEST_LOG =~ dump ]]; then
 				cat "$RESULTS"
 			fi
-			_fail "test failed for case $(basename $testname)"
+			_fail "test failed for case $testname"
 		fi
-		# These tests have overriden check_image() and their images may
+		# These tests have overridden check_image() and their images may
 		# have intentional unaligned metadata to trigger subpage
 		# warnings (like fsck/018), skip the check for their subpage
 		# warnings.
@@ -97,3 +112,5 @@ for i in $(find "$TEST_TOP/fsck-tests" -maxdepth 1 -mindepth 1 -type d	\
 do
 	run_one_test "$i"
 done
+
+"$TEST_TOP/scan-results.sh" "$RESULTS"
